package algorithm0302;

import static util450.Util450.*;

/**
 * Algorithm 3.2 of M. Ben-Ari, <i>Principles of Concurrent and Distributed
 * Programming</i>, Second edition, Addison Wesley, 2006.
 *
 * <p>
 * Implementation of the first attempt to solve the critical section problem for
 * two processes.
 *
 * <p>
 * File: <code>Algorithm0302.java</code>
 *
 * @author J. Stanley Warford
 */

public class Algorithm0302 extends Thread {

   static volatile int n = 0;
   static volatile int turn = 1;
   int processID;

   Algorithm0302(int pID) {
      processID = pID;
   }

   public void run() {
      int temp;
      if (processID == 1) { // Process p
         for (int i = 0; i < 10; i++) {
            try {
               randomDelay(10);
               // Preprotocol
               while (turn != 1) {
                  ;
               }
               // Critical section
               temp = n;
               randomDelay(10);
               n = temp + 1;
               // Postprotocol
               turn = 2;
            } catch (InterruptedException e) {
            }
         }
      } else if (processID == 2) { // Process q
         for (int i = 0; i < 10; i++) {
            try {
               randomDelay(10);
               // Preprotocol
               while (turn != 2) {
                  ;
               }
               // Critical section
               temp = n;
               randomDelay(10);
               n = temp + 1;
               // Postprotocol
               turn = 1;
            } catch (InterruptedException e) {
            }
         }
      }
   }

   public static void main(String[] args) {
      Algorithm0302 p = new Algorithm0302(1);
      Algorithm0302 q = new Algorithm0302(2);
      p.start();
      q.start();
      try {
         p.join();
         q.join();
      } catch (InterruptedException e) {
      }
      System.out.println("The value of n is " + n);
   }
}
